#!/bin/bash

sizes=(16 24 32 48 64 128 256)

DIRS=$(find * -maxdepth 0 -type d)

for theme_dir in $DIRS
do
	cnt_updated=0
	cnt_unchanged=0

	echo
	echo 'Generating "'$theme_dir'" theme'
	cd $theme_dir
	cd 'scalable'
	subdirs=$(find * -maxdepth 0 -type d)
	cd ..
	for size in "${sizes[@]}"
	do
		for subdir in $subdirs
		do
			input_dir='scalable'/$subdir/
			cd $input_dir
			icons=$(find *.svg -maxdepth 0 -type f)
			cd ../../
			size_dir=$size'x'$size
			output_dir=$size_dir/$subdir/
			mkdir -p $output_dir
			for icon in $icons
			do
				f_in=$input_dir$icon
				f_out=$output_dir${icon/.svg/.png}
				if [ $f_in -nt $f_out ]; then
					echo
					echo "Generating $theme_dir/$f_out ..."
					inkscape --export-filename=$f_out -w $size -h $size $f_in
					echo "... done."
					((cnt_updated++))
				else
					((cnt_unchanged++))
				fi
			done
		done
	done

	if [ "$cnt_updated" -gt "0" ]; then
		echo
		echo "Creating index.theme"

		theme_name="neobotix-$theme_dir"

		echo -e "[Icon Theme]\nName=$theme_name" > index.theme
		if [[ "$theme_name" != 'neobotix-hicolor' ]]; then
			echo -e 'Inhertits=neobotix-hicolor' >> index.theme
		fi
		echo -n "Directories=" >> index.theme

		dirs_fixed=$(find * -type d | grep -v git | grep -v scalable | grep "/" | sort -r)
		dirs_scalable=$(find * -type d | grep -v git | grep scalable | grep "/" | sort -r)

		for foo in $dirs_fixed
		do
			echo -n "$foo," >> index.theme
		done

		for foo in $dirs_scalable
		do
			echo -n "$foo," >> index.theme
		done

		for foo in $dirs_fixed
		do
			size=$(echo $foo | sed 's/\x.*//')

			ctx=$(basename $foo)
			if [[ "$ctx" == 'actions' ]]; then
				context='Context=Actions'
			elif [[ "$ctx" == 'apps' ]]; then
				context='Context=Applications'
			elif [[ "$ctx" == 'emblems' ]]; then
				context='Context=Emblems'
			fi

			type="Fixed"

			echo -en "\n\n[$foo]\n$context\nSize=$size\nType=$type" >> index.theme
		done

		for foo in $dirs_scalable
		do
			ctx=$(basename $foo)
			if [[ "$ctx" == 'actions' ]]; then
				context='Context=Actions'
			elif [[ "$ctx" == 'apps' ]]; then
				context='Context=Applications'
			elif [[ "$ctx" == 'emblems' ]]; then
				context='Context=Emblems'
			fi

			type="Scalable"

			size="Size=512"
			maxsize="MaxSize=512"

			echo -en "\n\n[$foo]\n$context\n$size\n$maxsize\nType=$type" >> index.theme
		done
	fi

	cd ..

	echo
	echo '"'$theme_dir'" summary:'
	echo '- - -'
	echo '  updated:   '$cnt_updated
	echo '  unchanged: '$cnt_unchanged
	echo '- - -'
	echo

done

